"""PyECDAR, a tool for timed games and timed specifications.
Copyright (C) INRIA Rennes - Louis-Marie Traonouez

Louis-Marie.Traonouez@inria.fr

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA."""

import pyppl
import re
import sys
import logging

logging.basicConfig()
logger = logging.getLogger('TIGA_PARAM')


def parse_expr(expr_str,delta_name):
  coeffs = [0,0]
  expr = re.search('^\s*(?P<coeff>(-)?\d+)?\s*(?P<str>.*)$',expr_str)
  expr_str = expr.group("str")
  if expr.group("coeff"):
    coeffs[0] = int(expr.group("coeff"))
    expr = re.search('^\s*(?P<coeff>(-|\+)?\d+)?\s*(?P<str>.*)$',expr_str)
    expr_str = expr.group("str")
    if expr.group("coeff"):
      coeffs[1] = int(expr.group("coeff"))
      
  if "*" in expr_str and coeffs[1] == 0:
    coeffs[1] = coeffs[0]
    coeffs[0] = 0
  
  if not coeffs[1] and delta_name in expr_str:
    if "-" in expr_str:
      coeffs[1] = -1
    else:
      coeffs[1] = 1
      
  return coeffs

def parse_param_invariant_or_guard(guard_str,delta_name,context):
  """ Parse a parametric guard into a ParamFederation domain. The guard syntax must be of the following:
  - only one parameter (delta_name)
  - x op coeff1 +/- coeff2 * delta_name or coeff1 +/- coeff2 * delta_name op x, 
  - where x is a clock,
  - coeff1 if present is an integer (possibly negative)
  - coeff2 if present is a positive integer
  - op is one of the following operators: {<,<=,==,>=,>}
  Return the ParamFederation corresponding to this guard.
  """
  if guard_str == "":
    return context.getTautologyParamFederation()
        
  param_variable = 0 # variable of the parameter delta: 0 by convention
  guard_domain = context.getTautologyParamFederation()
  for constraint_str in guard_str.split('&&'):
    constraint_expr = re.match('^\s*(?P<clock>\w+)\s*(?P<sign>((>=)|(>)|(==)|(<=)|(<)))\s*(?P<expr>.*)$',constraint_str)
    if constraint_expr:
      if constraint_expr.group("sign") == "<":
	sign = -2
      elif constraint_expr.group("sign") == "<=":
	sign = -1
      elif constraint_expr.group("sign") == "==":
	sign = 0
      elif constraint_expr.group("sign") == ">=":
	sign = 1
      elif constraint_expr.group("sign") == ">":
	sign = 2
    else:
      constraint_expr = re.search('^\s*(?P<expr>(\W|+|-|\*)+)\s*(?P<sign>((>=)|(>)|(==)|(<=)|(<)))\s*(?P<clock>\w+)\s*$',constraint_str)
      if constraint_expr:
	if constraint_expr.group("sign") == ">":
	  sign = -2
	elif constraint_expr.group("sign") == ">=":
	  sign = -1
	elif constraint_expr.group("sign") == "==":
	  sign = 0
	elif constraint_expr.group("sign") == "<=":
	   sign = 1	  
	elif constraint_expr.group("sign") == "<":
	   sign = 2
      else:
	logger.error("can't parse parametric guard " + guard_str)
	continue

    clock_variable = context.getIdVariable(constraint_expr.group("clock"))
    coeffs = parse_expr(constraint_expr.group("expr"),delta_name)
    guard_domain.addConjunction(pyppl.ParamGuard(clock_variable,param_variable,coeffs[0],coeffs[1],sign))
  return guard_domain

    
 
